%{
#include <string>
#include <list>
#include "my_parser/my_parser_includes.h"
#include <iostream>
#include <sstream>

std::string my_parser_current_identifier;

std::list<double> parsed_numbers;
std::list<std::string> parsed_identifiers;
%}

%union {
  std::string*       text;
  float*            number;
}

%token <text>                    IDENTIFIER
%token <number>                  NUMBER
%token                           END_OF_FILE

%% 


number_or_identifier_list :
  number_or_identifier eof {
#ifdef my_parser_debug
    my_parser_debug_string += "parsed a number or identifier\n";
#endif // my_parser_debug
  } |
  number_or_identifier_list number_or_identifier eof {
#ifdef my_parser_debug
    my_parser_debug_string += "parsed multiple numbers and identifiers.\n";
#endif // my_parser_debug
  }

eof:
  END_OF_FILE {
#ifdef my_parser_debug
    my_parser_debug_string += "parsed the end of the file.\n";
#endif // my_parser_debug
  }


number_or_identifier:
  NUMBER {
#ifdef my_parser_debug
    my_parser_debug_string += "parsed a number: ";

    std::ostringstream temp_string;
    temp_string << *$1;
    my_parser_debug_string += temp_string.str();

    my_parser_debug_string += "\n";
#endif // my_parser_debug

    parsed_numbers.push_back(*$1);
    delete $1;
  } |
  IDENTIFIER {
#ifdef my_parser_debug
    my_parser_debug_string += "parsed an identifier: ";

    std::ostringstream temp_string;
    temp_string << *$1;
    my_parser_debug_string += temp_string.str();

    my_parser_debug_string += "\n";
#endif // my_parser_debug
    parsed_identifiers.push_back(*$1);

    delete $1;
  }


/* ----------------------------------------------------------------- */

%% 

extern int my_parser_lineno;
extern int my_parser_lex();

#ifdef MY_PARSER_DEBUG
extern std::string my_parser_debug_string;
#endif // MY_PARSER_DEBUG

extern void my_parser_error(std::string s);
extern void my_parser_scanner_initialize();

void my_parser_parser_initialize()
{
  my_parser_scanner_initialize();

  my_parser_lineno = 1;
  parsed_numbers.clear();
  parsed_identifiers.clear();
}

/*
Sample main file

#include <string>
#include <stdio.h>
#include "my_parser/my_parser_includes.h"
#include "y.tab.h"

extern FILE *yyin;
extern int yyparse();
extern void yyinitialize();

int main(int argc, char *argv[]) {
  yyin = fopen(argv[1], "r");
  yyinitialize();
  yyparse();
  fclose(yyin);

  return 0;
}
*/
